  /**
    * @file SignalRelayPIC.c
    * @brief This module contains the implementation of the signal relay hardware
    * functions.
    *
    * @author Stefan Popovic
    *
    * @date 15/10/2016
    *
    * @version 0.1a
    *
    * @copyright GNU Public License
    *
    * Test configuration:
    *
    * MCU: P18F45K22
    * Dev.Board: EasyPIC v7 Connectivity
    * External Clock 32.0000 MHz
    * Ext. Modules: Signal Relay Click Board
    * SW: MikroC PRO for PIC v6.6.3
    *
    * NOTES:
      - Signal Relay click board is essentially board with 4 5VDC Signal Relays
      - Place Signal Relay click board on mikroBUS slot 1
      - Put button jumper (J17) into VCC position and pull-down pins 
        PORTD3, PORTD2, PORTD1 and PORTD0
      - RE 1: RA2
      - RE 2: RE1
      - RE 3: RE0
      - RE 4: RC0


    /**************************************************************************
    * Module Preprocessor Constants
    **************************************************************************/


    /**************************************************************************
    * Module Preprocessor Macros
    **************************************************************************/


    /**************************************************************************
    * Module Typedefs
    **************************************************************************/


     /**************************************************************************
     * Includes
     **************************************************************************/
     #include "built_in.h"
     #include <stdbool.h>

    /**************************************************************************
    * Module Variable Definitions
    **************************************************************************/

    char txt[24];
    char txt1[] = "RE1";
    char txt2[] = "RE2";
    char txt3[] = "RE3";
    char txt4[] = "RE4";
    bool oldD3;
    bool oldD2;
    bool oldD1;
    bool oldD0;

 // Lcd module connections
    sbit LCD_RS at LATB4_bit;
    sbit LCD_EN at LATB5_bit;
    sbit LCD_D4 at LATB0_bit;
    sbit LCD_D5 at LATB1_bit;
    sbit LCD_D6 at LATB2_bit;
    sbit LCD_D7 at LATB3_bit;

    sbit LCD_RS_Direction at TRISB4_bit;
    sbit LCD_EN_Direction at TRISB5_bit;
    sbit LCD_D4_Direction at TRISB0_bit;
    sbit LCD_D5_Direction at TRISB1_bit;
    sbit LCD_D6_Direction at TRISB2_bit;
    sbit LCD_D7_Direction at TRISB3_bit;
    // End Lcd module connections

   void Display_Init()
  {
     Lcd_Init();                                     // Initialize Lcd
     Lcd_Cmd(_LCD_CLEAR);                            // Clear display
     Lcd_Cmd(_LCD_CURSOR_OFF);                       // Cursor off

     Lcd_Out(1, 1, txt1);
     Lcd_Out(1, 10, txt2);
     Lcd_Out(2, 1, txt3);
     Lcd_Out(2, 10, txt4);

     Lcd_Out(1, 5, "OFF");                           // RE1
     Lcd_Out(1, 14, "OFF");                          // RE2
     Lcd_Out(2, 5, "OFF");                           // RE3
     Lcd_Out(2, 14, "OFF");                          // RE4
   }

    void main()
    {
      Display_Init();

      oldD3 = false;
      oldD2= false;
      oldD1 = false;
      oldD0 = false;
      
      ANSELA = 0;                                    // Configure AN pins as digital
      ANSELB = 0;
      ANSELC = 0;
      ANSELD = 0;
      
      TRISD3_bit = 1;                                // set RD3 pin as input
      TRISD2_bit = 1;                                // set RD2 pin as input
      TRISD1_bit = 1;                                // set RD1 pin as input
      TRISD0_bit = 1;                                // set RD0 pin as input
      
      TRISA2_bit = 0;                                // set RA2 pin as output
      TRISE1_bit = 0;                                // set RE1 pin as output
      TRISE0_bit = 0;                                // set RE0 pin as output
      TRISC0_bit = 0;                                // set RC0 pin as output
      
      LATA = 0x00;                                   // Initial PORTA value
      LATC = 0x00;                                   // Initial PORTC value
      LATE = 0x00;                                   // Initial PORTE value
      
      while(1) 
      {

          // Relay 1
          if (Button(&PORTD, 3, 1, 1))               // detect logical one on PORTA15 pin
            {

              if (!oldD3)                            // detect zero-to-one transition
              {
                Delay_ms(100);
                Lcd_Out(1, 5, "    ");
                Lcd_Out(1, 5, "ON");                 // RE1
                oldD3 = 1;
                LATA2_bit = 1;
              }
              else                                   // detect one-to-zero transition
              {
                Delay_ms(100);
                Lcd_Out(1, 5, "    ");
                Lcd_Out(1, 5, "OFF");                // RE1
                oldD3 = 0;
                LATA2_bit = 0;
              }
            }

          // Relay 2
          if (Button(&PORTD, 2, 1, 1))               // detect logical one on PORTC4 pin
            {

              if (!oldD2)                            // detect zero-to-one transition
              {
                Delay_ms(100);
                Lcd_Out(1, 14, "    ");
                Lcd_Out(1, 14, "ON");                // RE2
                oldD2 = 1;
                LATE1_bit = 1;
              }
              else                                   // detect one-to-zero transition
              {
                Delay_ms(100);
                Lcd_Out(1, 14, "    ");
                Lcd_Out(1, 14, "OFF");               // RE2
                oldD2 = 0;
                LATE1_bit = 0;
              }
            }

          // Relay 3
          if (Button(&PORTD, 1, 1, 1))               // detect logical one on PORTA14 pin
            {

              if (!oldD1)                            // detect zero-to-one transition
              {
                Delay_ms(100);
                Lcd_Out(2, 5, "    ");
                Lcd_Out(2, 5, "ON");                 // RE3
                oldD1 = 1;
                LATE0_bit = 1;
               }
              else                                   // detect one-to-zero transition
              {
                Delay_ms(100);
                Lcd_Out(2, 5, "    ");
                Lcd_Out(2, 5, "OFF");                // RE3
                oldD1 = 0;
                LATE0_bit = 0;
              }
            }

          // Relay 4
          if (Button(&PORTD, 0, 1, 1))               // detect logical one on PORTA14 pin
            {

              if (!oldD0)                            // detect zero-to-one transition
              {
                Delay_ms(100);
                Lcd_Out(2, 14, "    ");
                Lcd_Out(2, 14, "ON");                // RE4
                oldD0 = 1;
                LATC0_bit = 1;
              }
              else                                   // detect one-to-zero transition
              {
                Delay_ms(100);
                Lcd_Out(2, 14, "    ");
                Lcd_Out(2, 14, "OFF");               // RE4
                oldD0 = 0;
                LATC0_bit = 0;
              }
            }

      }

    }